// Communication type between the pages and the application base.
var comTypeEnum = 
{
	postMessage: 0 
};

// The message protocol in the inter pages communction.
var protocolTypeEnum = 
{
	rpc: 0,
	event: 1
};

/**
 * Base receiver object.
 * 
 * @author Yoel Gluschnaider
 */
var comUtils = (function () 
{
	var me = {};
	
	/**
	 * Parses the given message to an object and adds the meta data.
	 * @param {Object} msgStr
	 * @param {Number} type
	 * @param {Object} metaData
	 * 
	 * @return the message object
	 * 
	 * @author Yoel Gluschnaider
	 */
	me.parseMessage = function (msgStr, type, metaData) 
	{
		// un stringify the message to an object.
		var messageObj = JSON.parse(msgStr);
		
		// wrap the message with the meta data.
		var messageWrapper = 
		{
			msg: messageObj,
			type: type,
			metaData: metaData
		};
		
		return messageWrapper;
	};
	
	/**
	 * Create a string from the message and add the sender id.
	 * @param {Object} messageObj - the message object.
	 * 
	 * @return the message string.
	 */	
	me.stringifyMessage = function (messageObj) 
	{
		// Stringify the message to JSON string.
		var msgStr = JSON.stringify(messageObj);
		return msgStr;
	};
	
	return me;
})();


/**
 * Message receiver that listens to post messages.
 * 
 * @author Yoel Gluschnaider
 */
var PostMsgReceiver = function (cbReceive) 
{
	
	var me = {};
	
	// the receive call back function.
	var m_receiveCb;
	
	/**
	 * The message event listener. adds data to the message
	 * and passes it to the receive call back.
	 * @param {Object} e - message receive event object.
	 * 
	 * @author Yoel Gluschnaider
	 */
	function eventListener (e) 
	{
		// get the message string.
		var messageStr = e.data;
		var type = comTypeEnum.postMessage;
		var metaData = e.source;
		// parse the message and add meta data.
		var messageObj = comUtils.parseMessage(messageStr, type, metaData);
		// call the receiver's call back.
		m_receiveCb(messageObj);
	}
	
	/**
	 * The initialization method.
	 * @param {Function} cbReceive - the call back to call upon message arrival.
	 * 					Signature: cbReveive(messageObj).
	 * 					messageObj = {
	 * 									type: <message type>,
	 * 									meta: <message meta data>,
	 * 									msg: <message obj>
	 * 								}
	 * 
	 * @author Yoel Gluschnaider
	 */
	function init(cbReceive) 
	{
		m_receiveCb = cbReceive;
		// set the event listener for the post message.
		window.addEventListener("message", eventListener, false);
	}
	
	/**
	 * Stops listening to to messages.
	 * 
	 * @author Yoel Gluschnaider
	 */
	me.stop = function () 
	{
		window.removeEventListener("message", eventListener, false);	
	};
	
	// initialize the receiver.
	init(cbReceive);
	
	return me;
	
};

/**
 * Sender that uses the Post Message mechanism.
 * 
 * @author Yoel Gluschnaider
 */
var PostMsgSender = function (receiverWin) 
{

	var me = {};
	
	// receiver window element.
	var m_receiver;
	
	/**
	 * Initialization method
	 * @param {Object} receiverWin - the receiver's window element.
	 * 
	 * @author Yoel Gluschnaider
	 */
	function init(receiverWin) 
	{
		m_receiver = receiverWin;
	}
	
	/**
	 * Sends the message object to the receiver.
	 * @param {Object} messageObj - the message object
	 * 
	 * @author Yoel Gluschnaider
	 */
	me.sendMessage = function (messageObj) 
	{
		// Stringify the message.
		var msgStr = comUtils.stringifyMessage(messageObj);
		// post the message
		m_receiver.postMessage(msgStr, '*');
	};
	
	// initializes the object.
	init(receiverWin);
	
	return me;
	
};

/**
 * Communication Slave Module.
 * Sits in each page.
 * @author Yoel Gluschnaider
 */
var comSlave = (function () 
{
	
	var me = {};
	
	// The handshake message string.
	var HANDSHAKE_MSG = 'HANDSHAKE';
	
	// The ACK message string
	var HANDSHAKE_ACK_MSG = 'ACK';
	
	// the receive call back.
	var m_recCb;
		
	// an array of all the channels receivers.
	var m_recArr = [];
	
	// the final receiver
	var m_receiver;

	// The final sender of this slave.
	var m_sender;
	
	// the senders array to send the handshake
	var m_sendersArr = [];
	
	// The id of this slave (set in the handshake process).
	var m_id = null;
	

	/**
	 * Stops all listeners and clears the array.
	 * 
	 * @author Yoel Gluschnaider
	 */
	function stopAllReceivers() 
	{
		var recArrLen = m_recArr.length;
		for (var i = 0; i < recArrLen; i++) 
		{
			m_recArr[i].stop();
		}
		// clear out the array.
		m_recArr = [];
	}
	
	/**
	 * Message receive call back.
	 * @param {Object} messageObj
	 * 
	 * @author Yoel Gluschnaider
	 */
	function receiverCb(messageObj) 
	{
		if (m_recCb) 
		{
			m_recCb(messageObj.msg);
		}
	}
	
	/**
	 * A listener call back for the handshake event
	 * @param {Object} messageObj - the handshake message.
	 * 
	 * @author Yoel Gluschnaider
	 */
	function ackListener(messageObj) 
	{
		// sanity check
		if (messageObj.msg.messageTxt !== HANDSHAKE_ACK_MSG) 
		{
			return;
		}
		
		// get the id of the slave.
		m_id = messageObj.msg.slaveId;
		
		// Stop listening to all channels.
		stopAllReceivers();
		
		// check the message type so we will know which sender to create.
		switch (messageObj.type) 
		{
			case comTypeEnum.postMessage:
				m_sender = new PostMsgSender(messageObj.metaData);
				m_receiver = new PostMsgReceiver(receiverCb);
			break;
		}

		// notify the page we are ready.
		m_readyCb(m_id);
	}
	
	/**
	 * Creates receivers to all channels.
	 * 
	 * @author Yoel Gluschnaider
	 */
	function listenToAllChannels() 
	{
		m_recArr.push(new PostMsgReceiver(ackListener));
	}
	
	/**
	 * Creates senders to all the channels
	 * 
	 * @author Yoel Gluschnaider
	 */
	function createSendersToAllChannels() 
	{
		// check that there is a parent before creating the post message sender.
		if (parent) 
		{
			m_sendersArr.push(new PostMsgSender(parent));
		}
	}
	
	/**
	 * Sends handshake message to all the channels
	 * 
	 * @author Yoel Gluschnaider
	 */
	function sendHandshakes() 
	{
		createSendersToAllChannels();
		
		// send handshakes to all channels
		var handShakeMessage = {messageTxt: HANDSHAKE_MSG};
		var arrLen = m_sendersArr.length;
		for (var i=0; i < arrLen; i++) 
		{
			m_sendersArr[i].sendMessage(handShakeMessage);
		}

		// clear the senders array after sending handshakes.
		m_sendersArr = [];		
	}
	
	/**
	 * Register on the receive call back.
	 * @param {Function} receiveCb - the receive call back.
	 * 
	 * @author Yoel Gluschnaider
	 */
	me.registerOnReceive = function(receiveCb) 
	{
		m_recCb = receiveCb;
	};
		
	/**
	 * Initializes a new slave.
	 * @param {Function} readyCb - call back when ready (established connection with the comMaster).
	 * 
	 * @author Yoel Gluschnaider
	 */
	me.init = function (readyCb) 
	{
		m_readyCb = readyCb;
		listenToAllChannels();
		sendHandshakes();
	};
	
	/**
	 * Sends message to the master.
	 * Adds the slave id to the message.
	 * @param {Object} messageObj - the message to send
	 * 
	 * @author Yoel Gluschnaider
	 */
	me.sendMessage = function (messageObj) 
	{
		m_sender.sendMessage({
			msg: messageObj,
			slaveId: m_id
		});
	};
	
	return me;
	
})();

/**
 * Receives message requests from the Events Handler or the RPC sender
 * and sends them to the comSlave.
 * 
 * @author Yoel Gluschnaider
 */
var slaveDemux = (function () 
{
	
	var me = {};
	
	/**
	 * Send a register slave on event message.
	 * @param {String} eventName - the event name.
	 */
	me.registerOnEvent = function (eventName) 
	{
		comSlave.sendMessage( 
		{
			protocol: protocolTypeEnum.event,
			eventCb: 'registerSlave',
			eventName: eventName
		});
	};
	
	/**
	 * Send a unregister slave on event message.
	 * @param {String} eventName - the event name.
	 */
	me.unregisterOnEvent = function (eventName) 
	{
		comSlave.sendMessage( 
		{
			protocol: protocolTypeEnum.event,
			eventCb: 'unregisterSlave',
			eventName: eventName
		});
	};

	/**
	 * Send a request of an RPC.
	 * @param {Object} slaveId - the target slave id.
	 * @param {Object} rpcResponse - the rpc response object.
	 */
	me.sendRpcRequest = function (rpcRequest) 
	{
		comSlave.sendMessage( 
		{
			protocol: protocolTypeEnum.rpc,
			rpcRequest: rpcRequest
		});
	};
	
	return me;
	
})();

/**
 * Sends RPC messages from the slave to the master and waits for a respons.
 * @author Yoel Gluschnaider
 */
var rpcSender = (function () 
{
	
	var me = {};
	
	// a running index of each function call
	var m_id = 0;
	
	// the array of function calls.
	var m_rpcArr = [];
	
	/**
	 * Send RPC call through the comSlave 
	 * @param {String} funcName - the function name.
	 * @param {Array} args - the function arguments.
	 * 
	 * @author Yoel Gluschnaider 
	 */
	function sendRpc(funcName, args) 
	{
		slaveDemux.sendRpcRequest(
		{
			funcName: funcName,
			args: args,
			id: m_id
		});
	}
	
	/**
	 * Receiving call back to handle the response from the RPC receiver.
	 * @param {Object} messageObj - the message from the receiver.
	 * 
	 * @author Yoel Gluschnaider 
	 */
	me.responseReceived = function(messageObj) 
	{
		m_rpcArr[messageObj.id][messageObj.cbName].apply(window, messageObj.args);
	};

	/**
	 * Invoke an RPC. After the cbFail u can add the method's arguments.
	 * @param {String} funcName - the function name.
	 * @param {Function} cbSuccess - the success call back.
	 * @param {Funciton} cbFail - the failure call back. 
	 * 
	 * @author Yoel Gluschnaider 
	 */
	me.invoke = function (funcName, cbSuccess, cbFail /* , arguments */) 
	{
		// save a handle to the success and failure call backs.		
		m_rpcArr[m_id] = 
		{
			cbSuccess: cbSuccess,
			cbFail: cbFail
		};
		
		// send RPC request.
		sendRpc(funcName, [].slice.call(arguments, 3));

		// increment the ID so the next RPC will have a different ID.
		m_id++;		
	};
	
	return me;
	
})();

/**
 * Handles events from the page base.
 * 
 * @author Yoel Gluschnaider
 */
var eventsHandler = (function () 
{
	
	var me = {};
	
	// The event handlers registered.
	var m_eventsHandlers = {};
	
	// A running index which acts like the handle for each
	// event registration.
	var m_handle = 0;
	
	/**
	 * Called when the slaveMux receives an event messge.
	 * @param {String} eventName - the event's name
	 * @param {Object} data - the data of the event.
	 * 
	 * @author Yoel Gluschnaider
	 */
	me.eventReceived = function (eventName, data) 
	{
		var eventHandlers = m_eventsHandlers[eventName];
		// if there are events handlers registered on this event, dispatch them. 
		if (eventHandlers) 
		{
			for (var i in eventHandlers) 
			{
				eventHandlers[i](eventName, data);
			}
		}
	};
	
	/**
	 * Register on an event. 
	 *  
	 * @param {Object} eventName - the event's name.
	 * @param {Object} eventCb - the event's handler: eventCb(eventName).
	 * 
	 * @return {Object} eventHandle - the event handle to unregister.
	 * 
	 * @author Yoel Gluschnaider
	 */
	me.registerOnEvent = function (eventName, eventCb) 
	{
		// save the current handle.
		var curHandle = m_handle;
		// if there is no event handlers on this event, create
		// the initial event handlers array.
		if (!m_eventsHandlers[eventName]) 
		{
			m_eventsHandlers[eventName] = {};
		}
		// push the event handler to the handlers array.
		m_eventsHandlers[eventName][curHandle] = eventCb;
		
		// increment the handle.
		m_handle++;
		
		// notify the master that we registered on an event.
		slaveDemux.registerOnEvent(eventName);
		
		return curHandle;
	};
	
	/**
	 * Unregisters from an event.
	 * @param {Object} eventName - the event name.
	 * @param {Object} eventHandle - the handle returned in the registration function.
	 */
	me.unregisterFromEvent = function (eventName, eventHandle) 
	{
		// delete the event handler CB.
		delete m_eventsHandlers[eventName][eventHandle];
		
		// notify the master that we unregistered from an event.
		slaveDemux.unregisterOnEvent(eventName);
	};
	
	return me;
	
})();

/**
 * Slave multiplexer that receives a message from the slaveCom
 * and directs it to the relevant component (RPC sender or Event Handler)
 * 
 * @author Yoel Gluschnaider
 */
var slaveMux = (function () 
{
	
	var me = {};
	
	/**
	 * The call back that handles new messages from the comSlave
	 * and sends them to the relevant handler (eventsHandler or rpcSender).
	 * @param {Object} messageObj - the message from the master.
	 * 
	 * @author Yoel Gluschnaider
	 */
	function receiveCb(messageObj) 
	{
		switch(messageObj.protocol) 
		{
			case protocolTypeEnum.rpc:
				rpcSender.responseReceived(messageObj.rpcResponse);
			break;
			case protocolTypeEnum.event:
				eventsHandler.eventReceived(messageObj.eventName, messageObj.eventData);
			break;
		}
	}
	
	/**
	 * Initialization function, register on the receive message.
	 * 
	 * @author Yoel Gluschnaider
	 */
	function init() 
	{
		comSlave.registerOnReceive(receiveCb);
	}
	
	// init immidiatly.
	init();
	
	return me;
	
})();

/**
 * Main API entry point.
 */
var conduitApi = 
{
	init: function (cbSuccess, cbFail) 
	{
		if (typeof conduitApi.__BCAPI !== 'undefined' &&
			typeof conduitApi.__BCAPI.init === 'function') 
		{
			return conduitApi.__BCAPI.init(cbSuccess, cbFail);
		}
	},

	/**
	 * Check if the given module name is supported.
	 * @param {String} moduleName
	 * @return {boolean} true if the module is supported, false otherwise.
	 * 
	 * @author Yoel Gluschnaider
	 */
	moduleSupported: function(moduleName)
	{
		if (typeof conduitApi.__BCAPI !== 'undefined' && 
			typeof conduitApi.__BCAPI[moduleName] !== 'undefined' &&
			typeof conduitApi.__BCAPI[moduleName].isSupported === 'function') 
		
		{
			var res = conduitApi.__BCAPI[moduleName].isSupported(); 
			return res;
		}
		return false;
	},
	
	platform: 
	{
		/**
		 * Check if the UI API is supported.
		 * 
		 * @return {boolean} true if it is supported and false otherwise.
		 */
		isSupported: function () 
		{
			return conduitApi.moduleSupported('platform');
		},
		
		/**
		 * Check if the given module name is supported.
		 * @param {String} moduleName
		 * @return {boolean} true if the module is supported, false otherwise.
		 * 
		 * @author Yoel Gluschnaider
		 */
		moduleSupported: function(moduleName)
		{
			var thisModuleName = 'platform';
			if (conduitApi.moduleSupported(thisModuleName) &&
				typeof conduitApi.__BCAPI[thisModuleName][moduleName] !== 'undefined' &&
				typeof conduitApi.__BCAPI[thisModuleName][moduleName].isSupported === 'function') 
			{
				return conduitApi.__BCAPI[thisModuleName][moduleName].isSupported();
			}
			return false;
		},
		
		/**
		 * Acceleromater API.
		 */
		accelerometer: 
		{
			/**
			 * Check if the accelerometer API is supported.
			 * 
			 * @return {boolean} true if it is supported and false otherwise.
			 */
			isSupported: function () 
			{
				return conduitApi.platform.moduleSupported('accelerometer');
			},
			
			/**
		     * Get the last acceleration data
			 * 
			 * @param {Object} cbSuccess - callback function that provides the Acceleration information.
			 * 							Signature: function cbSuccess(accelerationData)
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
			 * 
			 * @author Yoel Gluschnaider
			 */
			getLastAcceleration: function (cbSuccess, cbFail) 
			{
				if (conduitApi.platform.accelerometer.isSupported()) 
				{
					conduitApi.__BCAPI.platform.accelerometer.getLastAcceleration(cbSuccess, cbFail);
				}
				else
				{
					cbFail();
				}
			},
	
			 /**
		     * Start acceleration
		     * @param {function} cbSuccess - Function to be called upon success
		     * 		signature: function cbSuccess(accelerationHandler, accelerationData)
		     * 					accelerationHandler - to be used in stopAcceleration.
		     * 					accelerationData - contains:
		     * 							x: Amount of motion on the x-axis. Range [0, 1] (Number)
		     * 							y: Amount of motion on the y-axis. Range [0, 1] (Number)
		     * 							z: Amount of motion on the z-axis. Range [0, 1] (Number)
		     * 							timestamp: Creation timestamp in milliseconds. (DOMTimeStamp)
		     *  
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
			 * 
		     * @param {function} options - Options for the acceleration
		     *                  Currently support:
		     *                      frequency - interval of the cbSuccess to be called
			 *
			 * @author Yoel Gluschnaider
		     */
			startAcceleration: function (cbSuccess, cbFail, options) 
			{
				if (conduitApi.platform.accelerometer.isSupported()) 
				{
					conduitApi.__BCAPI.platform.accelerometer.startAcceleration(cbSuccess, cbFail, options);
				}
				else 
				{
					cbFail();
				}
			},
			
			/**
		     * Stop the acceleration tracking
		     * @param {function} cbSuccess - Function to be called upon success
		     * 		signature: function cbSuccess()
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
		     * @param {object} accelerationHandle - Acceleration object to stop tracking
			 *
			 * @author Yoel Gluschnaider
		     */
			stopAcceleration: function (cbSuccess, cbFail, accelerationHandle) 
			{
				if (conduitApi.platform.accelerometer.isSupported()) 
				{
					conduitApi.__BCAPI.platform.accelerometer.stopAcceleration(cbSuccess, cbFail, accelerationHandle);
				}
				else 
				{
					cbFail();
				}
			}
		},
	
		/**
		 * Camera API.
		 */
		camera: 
		{
			/**
			 * Check if the camera API is supported.
			 * 
			 * @return {boolean} - true if it is supported and false otherwise.
			 */
			isSupported: function () 
			{
				return conduitApi.platform.moduleSupported('camera');
			},
			
			
			/**
			 * The format of the return value
			 */
			DestinationType: 
			{
				DATA_URL : 0, // Return image as base64 encoded string
        		FILE_URI : 1  // Return image file URI
			},
			
			/**
			 * The source of the picture
			 */
			PictureSourceType: 
			{
				PHOTOLIBRARY : 0,
    			CAMERA : 1,
    			SAVEDPHOTOALBUM : 2
			},
	
			/**
		     * Take a picture
		     * 
		     * @param cbSuccess - Function to be called upon success
		     * 		Signature: function cbSuccess(imageData)
		     * 						The imageData is Base64 encoding of the image data, 
		     * 						or the image file URI, depending on options used.
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
		     * @param options - Options for the camera to capture an image
		     *                  Currently support:
		     *                      quality: Quality of saved image. Range is [0, 100].
		     *                      destinationType: Choose the format of the return value. Defined in conduitApi.camera.DestinationType (Number)		     *                      
		     *                      sourceType: Set the source of the picture. Defined in conduitApi.camera.PictureSourceType (Number)
		     *                      allowEdit: Allow simple editing of image before selection. (Boolean)
		     *
			 * @author Yoel Gluschnaider
		     */
			captureImage: function(cbSucecess, cbFail, options) 
			{
				if (conduitApi.platform.camera.isSupported()) 
				{
					conduitApi.__BCAPI.platform.camera.captureImage(cbSucecess, cbFail, options);
				}
			}
		},
	
		/**
		 * Device Information API.
		 */
		deviceInfo: 
		{
			/**
			 * Check if the Device Information API is supported.
			 * @return {boolean} true if it is supported and false otherwise.
			 * 
			 * @author Yoel Gluschnaider
			 */
			isSupported: function () 
			{
				return conduitApi.platform.moduleSupported('deviceInfo');
			},
			
			/**
			 * The values returned by the getDeviceInfo function
			 */
			deviceTypeEnum:
			{
				unknown:  0,
				iphone:   1 << 0,
				android:  1 << 1,
				rim: 	  1 << 2,
				winPhone: 1 << 3,
				symbian:  1 << 4,
				bada: 	  1 << 5,
				ipad:	  1 << 6
			},
			
			/**
			 * The platform we run in.
			 */
			platformEnum: 
			{
				// website device simulator.
				simulator: 0,
				// installed on the device.
				nativeApp: 1,
				// as a web application.
				webApp: 2	
			},
			
			/**
			 * Get device information
			 * 
		     * @param {function} cbSuccess - Function to be called upon success
		     * 		signature: function cbSuccess(deviceInfoObj)
		     * 					deviceInfoObj - contains:
		     * 							device: deviceTypeEnum value (e.g. deviceTypeEnum.android)
		     * 							platform: platformTypeEnum value (e.g. platformTypeEnum.nativeApp)
		     * 							version: operating system version (e.g. 2.1 in case of Android).
		     * 							uuid: device's Universally Unique Identifier
		     * 							language: ISO 639-1 language code (e.g. 'en' for English).
		     *  
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
			 * 
			 * @author Yoel Gluschnaider
			 */
			getDeviceInfo : function (cbSuccess, cbFail) 
			{
				if (conduitApi.platform.deviceInfo.isSupported()) 
				{
					conduitApi.__BCAPI.platform.deviceInfo.getDeviceInfo(cbSuccess, cbFail);
				}
			}
		},
	
		/**
		 * Geo Location API.
		 */
		geo: 
		{
			/**
			 * Check if the Geo Location API is supported.
			 * @return {boolean} true if it is supported and false otherwise.
			 */
			isSupported: function () 
			{
				return conduitApi.platform.moduleSupported('geo');
			},
			
			ErrorCodes:
			{
				UNKNOWN_ERROR: 0,
				PERMISSION_DENIED: 1,
				POSITION_UNAVAILABLE: 2,
				TIMEOUT: 3
			},
			
			/**
		     * Asynchronously aquires the current position.
			 *
		     * @param cbSuccess - Function to be called upon success
		     * 					  Signature: cbSuccess(position).
		     * 					  Param position contains:
		     * 						coords: A set of geographic coordinates
		     * 							latitude: Latitude in decimal degrees. (Number)
		     * 							longitude: Longitude in decimal degrees. (Number)
		     * 							altitude: Height of the position in meters above the ellipsoid. (Number)
		     * 							accuracy: Accuracy level of the latitude and longitude coordinates in meters. (Number)
		     * 							altitudeAccuracy: Accuracy level of the altitude coordinate in meters. (Number)
		     * 							heading: Direction of travel, specified in degrees counting clockwise relative to the true north. (Number)
		     * 							speed: Current ground speed of the device, specified in meters per second. (Number)
		     * 						timestamp: Creation timestamp for coords in milliseconds. (DOMTimeStamp)
		     * 
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail(error)
			 * 								Error: The error returned by the device - 
			 * 									code: One of the predefined error codes listed below:
			 * 										Error.PERMISSION_DENIED
			 * 										Error.POSITION_UNAVAILABLE
			 * 										Error.TIMEOUT
			 * 									message: Error message describing the details of the error encountered.
		     * @param options - Optional parameters to customize the retrieval of the geolocation.
		     * 					{ maximumAge: 3000, timeout: 5000, enableHighAccuracy: true };
		     *                  The supported options are:
		     *                      frequency: How often to retrieve the position in milliseconds. This option is not part of the W3C spec and will be removed in the future. maximumAge should be used instead. (Number) (Default: 10000)
		     *                      enableHighAccuracy: Provides a hint that the application would like to receive the best possible results. (Boolean)
		     *                      timeout: The maximum length of time (msec) that is allowed to pass from the call to geolocation.getCurrentPosition or geolocation.watchPosition until the corresponding geolocationSuccess callback is invoked. (Number)
		     *                      maximumAge: Accept a cached position whose age is no greater than the specified time in milliseconds. (Number)
		     *                      
		     *                  Android Quirks
		     *                  The Android 2.x simulators will not return a geolocation result unless the enableHighAccuracy option is set to true.
		     *                  { enableHighAccuracy: true }
			 * 
			 * @author Yoel Gluschnaider
		     */
	    	getLocation: function(cbSuccess, cbFail, options) 
			{
				if (conduitApi.platform.geo.isSupported()) 
				{
					conduitApi.__BCAPI.platform.geo.getLocation(cbSuccess, cbFail, options);
				}
			},
			
			/**
			 * Watches for changes to the device's current position.
		     * @param {function} cbSuccess - Function to be called upon success
		     * 		signature: function cbSuccess(geoHandler, position)
		     * 					geoHandle - to be used in stopGeoLocation.
		     * 					position - contains:
		     * 							coords: A set of geographic coordinates:
		     * 								latitude: Latitude in decimal degrees. (Number)
		     * 								longitude: Longitude in decimal degrees. (Number)
		     * 								altitude: Height of the position in meters above the ellipsoid. (Number)
		     * 								accuracy: Accuracy level of the latitude and longitude coordinates in meters. (Number)
		     *								altitudeAccuracy: Accuracy level of the altitude coordinate in meters. (Number)
		     * 								heading: Direction of travel, specified in degrees counting clockwise relative to the true north. (Number)
		     * 								speed: Current ground speed of the device, specified in meters per second. (Number)
		     * 								
		     * 							timestamp: Creation timestamp for coords in milliseconds. (DOMTimeStamp)  
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail(error)
			 * 								Error: The error returned by the device - 
			 * 									code: One of the predefined error codes listed below:
			 * 										Error.PERMISSION_DENIED
			 * 										Error.POSITION_UNAVAILABLE
			 * 										Error.TIMEOUT
			 * 									message: Error message describing the details of the error encountered.
			 * 
		     * @param options - Optional parameters to customize the retrieval of the geolocation.
		     * 					{ maximumAge: 3000, timeout: 5000, enableHighAccuracy: true };
		     *                  The supported options are:
		     *                      frequency: How often to retrieve the position in milliseconds. This option is not part of the W3C spec and will be removed in the future. maximumAge should be used instead. (Number) (Default: 10000)
		     *                      enableHighAccuracy: Provides a hint that the application would like to receive the best possible results. (Boolean)
		     *                      timeout: The maximum length of time (msec) that is allowed to pass from the call to geolocation.getCurrentPosition or geolocation.watchPosition until the corresponding geolocationSuccess callback is invoked. (Number)
		     *                      maximumAge: Accept a cached position whose age is no greater than the specified time in milliseconds. (Number)
		     *                      
		     *                  Android Quirks
		     *                  The Android 2.x simulators will not return a geolocation result unless the enableHighAccuracy option is set to true.
		     *                  { enableHighAccuracy: true }
			 */
			startGeoLocation : function (cbSuccess, cbFail, options) 
			{
				if (conduitApi.platform.geo.isSupported()) 
				{
					conduitApi.__BCAPI.platform.geo.startGeoLocation(cbSuccess, cbFail, options);
				}
			},
	
			/**
		     * @param {function} cbSuccess - Function to be called upon success
		     * 		signature: function cbSuccess()
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
			 * @param {Object} geoHandle - retrieved from the startGeoLocation function
			 */
			stopGeoLocation : function (cbSuccess, cbFail, geoHandle) 
			{
				if (conduitApi.platform.geo.isSupported()) 
				{
					conduitApi.__BCAPI.platform.geo.stopGeoLocation(geoHandle, cbSuccess, cbFail);
				}
			}
		},
	
		/**
		 * Network API.
		 */
		network: 
		{
			/**
			 * Check if the Network API is supported.
			 * @return {boolean} true if it is supported and false otherwise.
			 */
			isSupported: function () 
			{
				return conduitApi.platform.moduleSupported('network');
			},
			
			/**
		     * Send http request
		     * @param cbSuccess - Function to be called upon success
		     * 			Signature: cbSuccess(data, requestMetaData)
		     * 					data - The data returned from the server, formatted according to the 'dataType' parameter.
		     * 					requestMetaData - same as the requestMetaData parameter.
			 * @param {function} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail(requestMetaData)
		     * @param sMethod - Method of request: GET, PUT. default is GET
		     * @param sUrl - Request URL
		     * @param sParams - Request body (for PUT)
		     * @param bAsync - Make asynchronous request
		     * @param sDataType - Type of response data. default is JSON
		     * @param sUser - User name for authenticated request
		     * @param sPassword - Password for authenticated request
		     * @param requestMetaData - Data to be passed back to callbacks
		     *
			 * @author Yoel Gluschnaider
		     */
			crossDomainHttpRequest: function (cbSuccess, cbFail, sMethod, sUrl, sParams, bAsync, sDataType, sUser, sPassword, requestMetaData) 
			{
				if (conduitApi.platform.network.isSupported()) 
				{
					conduitApi.__BCAPI.platform.network.crossDomainHttpRequest(sMethod, sUrl, sParams, bAsync, sDataType, cbSuccess, cbFail, sUser, sPassword, requestMetaData);
				}
			}
		},
	
		/**
		 * Screen Orientation API.
		 */
		orientation: 
		{
			/**
			 * Check if the Screen Orientation API is supported.
			 * @return {boolean} true if it is supported and false otherwise.
			 */
			isSupported: function () 
			{
				return conduitApi.platform.moduleSupported('orientation');
			},
			
			/**
		     * Get the current orientation
		     * @param cbSuccess - Function to be called upon success
		     * 					  Signature: cbSuccess(curOrientation).
		     * 					  Param position contains:
		     * 						curOrientation: signed integer of device orientation 
		     * 										degree from base can be: 0, 90, 180, -90
		     * 
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
			 *
			 * @author Yoel Gluschnaider
		     */ 
		    
			getOrientation: function (cbSuccess, cbFail) 
			{
				if (conduitApi.platform.orientation.isSupported()) 
				{
					conduitApi.__BCAPI.platform.orientation.getOrientation(cbSuccess, cbFail);
				}
			},
			
			/**
		     * Lock device on the orientation
		 	 *
		     * @param cbSuccess - Function to be called upon success
		     * 					  Signature: function cbSuccess();
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
			 * @param degree - signed integer of device orientation degree from base
			 *				  can be: 0, 90, 180, -90
			 *
			 * @author Yoel Gluschnaider
		     */ 
			lockOrientation: function (cbSuccess, cbFail, degree) 
			{
				if (conduitApi.platform.orientation.isSupported()) 
				{
					conduitApi.__BCAPI.platform.orientation.lockOrientation(cbSuccess, cbFail, degree);
				}
			},
			
			/**
		     * Unlock device on the orientation
		     * @param cbSuccess - Function to be called upon success
		     * 					  Signature: function cbSuccess();
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
		 	 *
			 * @author Yoel Gluschnaider
			 */ 
			unlockOrientation: function (cbSuccess, cbFail) 
			{
				if (conduitApi.platform.orientation.isSupported()) 
				{
					conduitApi.__BCAPI.platform.orientation.unlockOrientation(cbSuccess, cbFail);
				}
			},
	
			/**
		     * Register event handler for orientation changed
			 *
		     * @param cbSuccess - Orientation change event handler
		     * 							Signature: cbOrientationChanged(degrees)
		     * 								degrees: 0, 90, 180 or -90.
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
			 *
			 * @author Yoel Gluschnaider
		     */
		    startOrientation: function (cbSuccess, cbFail) 
			{
				if (conduitApi.platform.orientation.isSupported()) 
				{
					conduitApi.__BCAPI.platform.orientation.startOrientation(cbSuccess, cbFail);
				}
			},
			
			/**
		     * Unregister orientation changed event
			 * 
		     * @param cbSuccess - Function to be called upon success
		     * 					  Signature: function cbSuccess();
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
			 */
			stopOrientation: function (cbSuccess, cbFail) 
			{
				if (conduitApi.platform.orientation.isSupported()) 
				{
					conduitApi.__BCAPI.platform.orientation.stopOrientation(cbSuccess, cbFail);
				}			
			}
		},
	
		/**
		 * Push Notification API.
		 */
		pushNotification: 
		{
			/**
			 * Check if the Push Notification API is supported.
			 * @return {boolean} true if it is supported and false otherwise.
			 */
			isSupported: function () 
			{
				return conduitApi.platform.moduleSupported('pushNotification');
			},
	
			/**
		     * Register component for push notification
		     * @param cbSuccess - Function to be called when the user chose to view the notification.
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
			 *
			 * @author Yoel Gluschnaider
		     */
		    registerPushNotification: function (cbSuccess, cbFail, cbEventHandler) 
			{
				if (conduitApi.platform.pushNotification.isSupported()) 
				{
					conduitApi.__BCAPI.platform.pushNotification.registerPushNotification(
							cbSuccess, cbFail, cbEventHandler);
				}
			},
			
			/**
		     * Remove registeration for a component
		     * @param {Function} cbSuccess - Function to be called upon success
		     * 					  		Signature: function cbSuccess();
			 * @param {Function} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
			 *
			 * @author Yoel Gluschnaider
		     */
		    unregisterPushNotification: function (cbSuccess, cbFail) 
			{
				if (conduitApi.platform.pushNotification.isSupported()) 
				{
					conduitApi.__BCAPI.platform.pushNotification.unregisterPushNotification(cbSuccess, cbFail);
				}
			}
		},
	
		/**
		 * Local Storage API.
		 */
		storage: 
		{
			/**
			 * Check if the accelerometer API is supported.
			 * @return {boolean} true if it is supported and false otherwise.
			 */
			isSupported: function () 
			{
				return conduitApi.platform.moduleSupported('storage');
			},
	
		    /**
		     * Save a value for a key under an identifier
		     * @param {Function} cbSuccess - Function to be called upon success
		     * 					  		Signature: function cbSuccess();
			 * @param {Function} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
		     * @param key - Key name
		     * @param value - Value for the key
		     * 
		     * @return true if succeeded and false otherwise.
			 * @author Yoel Gluschnaider
		     */
			setItem: function (cbSuccess, cbFail, key, value) 
			{
				if (conduitApi.platform.storage.isSupported()) 
				{
					return conduitApi.__BCAPI.platform.storage.setItem(cbSuccess, cbFail, key, value);
				}
			},
			
		    /**
		     * Get value for a key under an identifier
		     * 
		     * @param key - Key name
		     * @param {Function} cbSuccess - Function to be called upon success
		     * 					  		Signature: function cbSuccess(value);
			 * @param {Function} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
		     * @return {object} the value or null if none exists.
		     *
			 * @author Yoel Gluschnaider
		     */
			getItem: function (cbSuccess, cbFail, key)
			{
				if (conduitApi.platform.storage.isSupported()) 
				{
					return conduitApi.__BCAPI.platform.storage.getItem(cbSuccess, cbFail, key);
				}
			},
			
		    /**
		     * Remove a key under an identifier
		     * @param {Function} cbSuccess - Function to be called upon success
		     * 					  		Signature: function cbSuccess();
			 * @param {Function} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
		     * @param key - Key name
		     * @return true if succeeded and false otherwise.
		     *
			 * @author Yoel Gluschnaider
		     */
			removeItem: function (cbSuccess, cbFail, key) 
			{
				if (conduitApi.platform.storage.isSupported()) 
				{
					return conduitApi.__BCAPI.platform.storage.removeItem(cbSuccess, cbFail, key);
				}
			},
			
		    /**
		     * Clear the whole storage for an identifier
		     * @param {Function} cbSuccess - Function to be called upon success
		     * 					  		Signature: function cbSuccess();
			 * @param {Function} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
		     * @return true if succeeded and false otherwise.
		     *
			 * @author Yoel Gluschnaider
		     */
			clearStorage: function (cbSuccess, cbFail) 
			{
				if (conduitApi.platform.storage.isSupported()) 
				{
					return conduitApi.__BCAPI.platform.storage.clearStorage(cbSuccess, cbFail);
				}
			}
		},
	
		/**
		 * Vibration API.
		 */
		vibrate: 
		{
			/**
			 * Check if the Vibration API is supported.
			 * @return {boolean} true if it is supported and false otherwise.
			 */
			isSupported: function () 
			{
				return conduitApi.platform.moduleSupported('vibrate');
			},
	
		    /**
		     * Vibrate device for a specific time
			 *
		     * @param {Function} cbSuccess - Function to be called upon success
		     * 					  		Signature: function cbSuccess();
			 * @param {Function} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
			 * @param nTime - Time in miliseconds for the vibration
			 *					nTime range from 0 to 10000. default is 10
		     *
			 * @author Yoel Gluschnaider
		     */
			vibrateForTime: function (cbSuccess, cbFail, nTime) 
			{
				if (conduitApi.platform.vibrate.isSupported()) 
				{
					conduitApi.__BCAPI.platform.vibrate.vibrateForTime(cbSuccess, cbFail, nTime);
				}
			}
	
		},
		
		/**
		 * Gestures API.
		 */
		gestures: 
		{
			/**
			 * Check if the Gestures API is supported.
			 * @return {boolean} true if it is supported and false otherwise.
			 */
			isSupported: function () 
			{
				return conduitApi.platform.moduleSupported('gestures');
			},
			
			/**
		     * Register component for shake gesture
			 *
		     * @param cbSuccess - Function to be called upon success
		     * 					  Signature: function cbSuccess();
			 * @param cbFail - callback function in case the API callback failed.
			 *					  Signature: function cbFail()
		     * @param cbEventHandler - Function to be called when the user shake the device
			 *
			 * @author Moshe Darzi
		     */
			registerShake: function (cbSuccess, cbFail, cbEventHandler) 
			{
				if (conduitApi.platform.gestures.isSupported()) 
				{
					conduitApi.__BCAPI.platform.gestures.registerShake(cbSuccess, cbFail, cbEventHandler);
				}
			},
			
			/**
		     * Remove registeration for a component
			 *
		     * @param cbSuccess - Function to be called upon success
		     * 					  Signature: function cbSuccess();
			 * @param cbFail - callback function in case the API callback failed.
			 *					  Signature: function cbFail()
			 * @author Moshe Darzi
		     */
			unregisterShake: function (cbSuccess, cbFail) 
			{
				if (conduitApi.platform.gestures.isSupported()) 
				{
					conduitApi.__BCAPI.platform.gestures.unregisterShake(cbSuccess, cbFail);
				}
			}
		}
		
	},
	
	UI: 
	{
		/**
		 * Check if the UI API is supported.
		 * 
		 * @return {boolean} true if it is supported and false otherwise.
		 */
		isSupported: function () 
		{
			return conduitApi.moduleSupported('UI');
		},
		
		/**
		 * Show an activity indicator.
		 * 
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @param {Object} message - the message to be shown in the Activity Indicator.
		 */
		showActivityIndicator: function (cbSuccess, cbFail, message) 
		{
			if (conduitApi.UI.isSupported()) 
			{
				conduitApi.__BCAPI.UI.showActivityIndicator(cbSuccess, cbFail, message);
			}
		},
		
		/**
		 * Dismiss the activity indicator.
		 * 
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 */
		dismissActivityIndicator: function (cbSuccess, cbFail) 
		{
			if (conduitApi.UI.isSupported()) 
			{
				conduitApi.__BCAPI.UI.dismissActivityIndicator(cbSuccess, cbFail, document.body);
			}
		},
		
		/**
		 * Show a dialog box.
		 * 
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @param {Object} The dialog options contains:
		 * 						title: the title of the dialog.
		 * 						message: the message of the dialog.
		 * 						buttons: an array containing Button objects that consist of:
		 * 								text: the text of the button.
		 * 								cb: the callback to run when the button is pressed.
		 */
		showDialogBox: function (cbSuccess, cbFail, dialogOptions) 
		{
			if (conduitApi.UI.isSupported()) 
			{
				conduitApi.__BCAPI.UI.showDialogBox(cbSuccess, cbFail, dialogOptions);
			}
		}
	},

	app: 
	{
		/**
		 * Check if the app API is supported.
		 * 
		 * @return {boolean} true if it is supported and false otherwise.
		 */
		isSupported: function () 
		{
			return conduitApi.moduleSupported('app');
		},
		
		/**
		 * Hide info bar.
		 * 
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @return {boolean} true if it is succeeded and false otherwise.
		 * 
		 * @author Moshe Darzi
		 */
		hideInfoBar: function (cbSuccess, cbFail) 
		{
			if (conduitApi.app.isSupported()) 
			{
				conduitApi.__BCAPI.app.hideInfoBar(cbSuccess, cbFail);
			}
		},
		
		/**
		 * Show info bar.
		 * 
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @return {boolean} true if it is succeeded and false otherwise.
	     * 
		 * @author Moshe Darzi
		 */
		showInfoBar: function (cbSuccess, cbFail) 
		{
			if (conduitApi.app.isSupported()) 
			{
				return conduitApi.__BCAPI.app.showInfoBar(cbSuccess, cbFail);
			}
		},
		
		/**
		 * Toggle visiblity of the info bar.
		 * 
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @return {boolean} true if it is succeeded and false otherwise.
	     * 
		 * @author Moshe Darzi
		 */
		toggleInfoBar: function (cbSuccess, cbFail) 
		{
			if (conduitApi.app.isSupported()) 
			{
				return conduitApi.__BCAPI.app.toggleInfoBar(cbSuccess, cbFail);
			}
		},

		/**
		 * Return the visiblity of the info bar.
		 * 
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess(visible);
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @return true if visible, false otherwise
	     * 
		 * @author Moshe Darzi
		 */
		isInfoBarVisible: function (cbSuccess, cbFail)
		{
			if (conduitApi.app.isSupported()) 
			{
				return conduitApi.__BCAPI.app.isInfoBarVisible(cbSuccess, cbFail);
			}
		},

		/**
		 * Hide navigation bar.
		 * 
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @return {boolean} true if it is succeeded and false otherwise.
	     * 
		 * @author Moshe Darzi
		 */
		hideNavigationBar: function (cbSuccess, cbFail) 
		{
			if (conduitApi.app.isSupported()) 
			{
				return conduitApi.__BCAPI.app.hideNavigationBar(cbSuccess, cbFail);
			}
		},
		
		/**
		 * Show navigation bar.
		 * 
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @return {boolean} true if it is succeeded and false otherwise.
	     * 
		 * @author Moshe Darzi
		 */
		showNavigationBar: function (cbSuccess, cbFail) 
		{
			if (conduitApi.app.isSupported()) 
			{
				return conduitApi.__BCAPI.app.showNavigationBar(cbSuccess, cbFail);
			}
		},
		
		/**
		 * Toggle visiblity of the navigation bar.
		 * 
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @return {boolean} true if it is succeeded and false otherwise.
	     * 
		 * @author Moshe Darzi
		 */
		toggleNavigationBar: function (cbSuccess, cbFail) 
		{
			if (conduitApi.app.isSupported()) 
			{
				return conduitApi.__BCAPI.app.toggleNavigationBar(cbSuccess, cbFail);
			}
		},
		
		/**
		 * Return the visiblity of the navigation bar.
		 * 
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess(visable);
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @return true if visible, false otherwise
	     * 
		 * @author Moshe Darzi
		 */
		isNavigationBarVisible: function (cbSuccess, cbFail)
		{
			if (conduitApi.app.isSupported()) 
			{
				return conduitApi.__BCAPI.app.isNavigationBarVisible(cbSuccess, cbFail);
			}
		},

		/**
		 * Add button to the info bar.
		 * 
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess(btnHandle);
	     * 						btnHandle - the button's handle for later removal.
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @param {Object} cbButton - the callback of the button.
		 * @param {Object} buttonImage - the image of the button.
		 * @param {bool} addToRight - true indicates the button will be added to the right side,
		 * 							false will add to the left side
		 * 
		 * @return {Object} the button's handle for later removal or null if the addition failed.
	     * 
		 * @author Moshe Darzi
		 */
		addInfoBarButton: function (cbSuccess, cbFail, cbButton, buttonImage, addToRight) 
		{
			if (conduitApi.app.isSupported()) 
			{
				return conduitApi.__BCAPI.app.addInfoBarButton(cbSuccess, cbFail, cbButton, buttonImage, addToRight);
			}
		},
		
		/**
		 * Remove button from the info bar.
		 * 
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @param {Object} buttonHandle - the handle of the button.
		 * 
		 * @return {boolean} true if it is succeeded and false otherwise.
	     * 
		 * @author Moshe Darzi
		 */
		removeInfoBarButton: function (cbSuccess, cbFail, buttonHandle) 
		{
			if (conduitApi.app.isSupported()) 
			{
				return conduitApi.__BCAPI.app.removeInfoBarButton(cbSuccess, cbFail, buttonHandle);
			}
		},
		
		/**
		 * Remove all buttons from the info bar.
		 * 
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @return {boolean} true if it is succeeded and false otherwise.
	     * 
		 * @author Moshe Darzi
		 */
		removeAllInfoBarButtons: function (cbSuccess, cbFail) 
		{
			if (conduitApi.app.isSupported()) 
			{
				return conduitApi.__BCAPI.app.removeAllInfoBarButtons(cbSuccess, cbFail);
			}
		},
		
		/**
		 * 
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @param {Object} args - contains the url and the title.
		 * 							{
		 * 								url: 'http://www.conduit.com',
		 * 								title: 'Conduit'
		 * 							}
	     * 
		 * @author Moshe Darzi
		 */
		openLink: function (cbSuccess, cbFail, args) 
		{
			if (conduitApi.app.isSupported()) 
			{
				conduitApi.__BCAPI.app.openLink(cbSuccess, cbFail, args);
				// Send usage statistic
				var usageMeta = { url: args.url};
				conduitApi.__BCAPI.analytics.addEvent(cbSuccess, cbFail, conduitApi.analytics.usageEventTypeEnum.OpenLink, usageMeta);
			}
		},
		
		/**
		 * Closes the application.
	     * 
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @author Moshe Darzi
		 */
		closeApp: function (cbSuccess, cbFail) 
		{
			if (conduitApi.app.isSupported()) 
			{
				conduitApi.__BCAPI.app.closeApp(cbSuccess, cbFail);
			}
		}
	},
	
	analytics:
	{
		/**
		 * Check if the analytics API is supported.
		 * 
		 * @return {boolean} true if it is supported and false otherwise.
		 */
		isSupported: function () 
		{
			return true;
		},
		
		/**
		 * To be used by the 'event' function.
		 */
		usageEventTypeEnum: 
		{
			Unknown: 0,
		    TabClick: 1, // Tab is clicked on the navigation bar
		    OpenLink: 2, // URL linked is opened
		    ReceivedPush: 3, // Push notification received
		    SocialLogin: 4,
		    SocialShare: 5,
		    ViewChanged: 6,
		    BannerClicked: 7, // Ad was clicked
		    UserLoggedIn: 8 // User logged with a new app (when the app is loading)
		},

		/**
		 * To be used by the social share 'event' function.
		 */
		socialShareTypes:
		{
		    Unknown: 0,
		    Facebook: 1,
		    Email: 2,
		    Twitter: 3
		},

		/**
		 * 
	     * Add event to analytics stack
		 *
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
	     * @param type - Type of events from usageEventTypeEnum
	     * @param meta - The event's meta data
	     * 
		 * @author Moshe Darzi
		 */
		addEvent: function (cbSuccess, cbFail, type, meta) 
		{
			if (conduitApi.analytics.isSupported()) 
			{
				conduitApi.__BCAPI.analytics.addEvent(cbSuccess, cbFail, type, meta);
			}
		}
	},
	
	log: 
	{
		/**
		 * Check if the log API is supported.
		 * 
		 * @return {boolean} true if it is supported and false otherwise.
		 */
		isSupported: function () 
		{
			return true;
		},
		
		/**
		 * Write a log message to the native logger.
		 * Verbosity - Info (if supported).
		 * 
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @param {Object} message the log message.
		 */
		info: function (cbSuccess, cbFail, message) 
		{
			if (conduitApi.log.isSupported()) 
			{
				conduitApi.__BCAPI.log.info(cbSuccess, cbFail, message);
			}
		},
		
		/**
		 * Write a log message to the native logger.
		 * Verbosity - warning (if supported).
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @param {Object} message the log message.
		 */
		warning: function (cbSuccess, cbFail, message) 
		{
			if (conduitApi.log.isSupported()) 
			{
				conduitApi.__BCAPI.log.warning(cbSuccess, cbFail, message);
			}
		},
		
		/**
		 * Write a log message to the native logger.
		 * Verbosity - error (if supported).
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @param {Object} message the log message.
		 */
		error: function (cbSuccess, cbFail, message) 
		{
			if (conduitApi.log.isSupported()) 
			{
				conduitApi.__BCAPI.log.error(cbSuccess, cbFail, message);
			}
		}
	}
	
};

conduitApi.__BCAPI = 
{
	init: function (cbSuccess, cbFail) 
	{
		// initialize this slave.
		comSlave.init(function (slaveId) 
		{
			// TODO: remove
			document._pageId = slaveId;
			function innerCbSuccess(supportedObj) 
			{
				conduitApi.__BCAPI.supportedObj = supportedObj;
				cbSuccess();
			}
			rpcSender.invoke('conduitApi.init', innerCbSuccess, cbFail);
		});
	},
	
	platform: 
	{
		/**
		 * Check if the UI API is supported.
		 * 
		 * @return {boolean} true if it is supported and false otherwise.
		 */
		isSupported: function () 
		{
			return conduitApi.__BCAPI.supportedObj.platform.isSupported;
		},
		
		/**
		 * Acceleromater API.
		 */
		accelerometer: 
		{
			/**
			 * Check if the accelerometer API is supported.
			 * 
			 * @return {boolean} true if it is supported and false otherwise.
			 */
			isSupported: function () 
			{
				return conduitApi.__BCAPI.supportedObj.platform.accelerometer.isSupported;
			},
			
			/**
		     * Get the last acceleration data
			 * 
			 * @param {Object} cbSuccess - callback function that provides the Acceleration information.
			 * 							Signature: function cbSuccess(accelerationData)
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
			 * 
			 * @author Yoel Gluschnaider
			 */
			getLastAcceleration: function (cbSuccess, cbFail) 
			{
				rpcSender.invoke('conduitApi.platform.accelerometer.getLastAcceleration', cbSuccess, cbFail);
			},
	
			 /**
		     * Start acceleration
		     * @param {function} cbSuccess - Function to be called upon success
		     * 		signature: function cbSuccess(accelerationHandler, accelerationData)
		     * 					accelerationHandler - to be used in stopAcceleration.
		     * 					accelerationData - contains:
		     * 							x: Amount of motion on the x-axis. Range [0, 1] (Number)
		     * 							y: Amount of motion on the y-axis. Range [0, 1] (Number)
		     * 							z: Amount of motion on the z-axis. Range [0, 1] (Number)
		     * 							timestamp: Creation timestamp in milliseconds. (DOMTimeStamp)
		     *  
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
			 * 
		     * @param {function} options - Options for the acceleration
		     *                  Currently support:
		     *                      frequency - interval of the cbSuccess to be called
			 *
			 * @author Yoel Gluschnaider
		     */
			startAcceleration: function (cbSuccess, cbFail, options) 
			{
				rpcSender.invoke('conduitApi.platform.accelerometer.startAcceleration', cbSuccess, cbFail, options);
			},
			
			/**
		     * Stop the acceleration tracking
		     * @param {function} cbSuccess - Function to be called upon success
		     * 		signature: function cbSuccess()
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
		     * @param {object} accelerationHandle - Acceleration object to stop tracking
			 *
			 * @author Yoel Gluschnaider
		     */
			stopAcceleration: function (cbSuccess, cbFail, accelerationHandle) 
			{
				rpcSender.invoke('conduitApi.platform.accelerometer.stopAcceleration', cbSuccess, cbFail, accelerationHandle);
			}
		},
	
		/**
		 * Camera API.
		 */
		camera: 
		{
			/**
			 * Check if the camera API is supported.
			 * 
			 * @return {boolean} - true if it is supported and false otherwise.
			 */
			isSupported: function () 
			{
				return conduitApi.__BCAPI.supportedObj.platform.camera.isSupported;
			},

			/**
		     * Take a picture
		     * 
		     * @param cbSuccess - Function to be called upon success
		     * 		Signature: function cbSuccess(imageData)
		     * 						The imageData is Base64 encoding of the image data, 
		     * 						or the image file URI, depending on options used.
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
		     * @param options - Options for the camera to capture an image
		     *                  Currently support:
		     *                      quality: Quality of saved image. Range is [0, 100].
		     *                      destinationType: Choose the format of the return value. Defined in conduitApi.camera.DestinationType (Number)		     *                      
		     *                      sourceType: Set the source of the picture. Defined in conduitApi.camera.PictureSourceType (Number)
		     *                      allowEdit: Allow simple editing of image before selection. (Boolean)
		     *
			 * @author Yoel Gluschnaider
		     */
			captureImage: function(cbSuccess, cbFail, options) 
			{
				rpcSender.invoke('conduitApi.platform.camera.captureImage', cbSuccess, cbFail, options);
			}
		},
	
		/**
		 * Device Information API.
		 */
		deviceInfo: 
		{
			/**
			 * Check if the Device Information API is supported.
			 * @return {boolean} true if it is supported and false otherwise.
			 * 
			 * @author Yoel Gluschnaider
			 */
			isSupported: function () 
			{
				return conduitApi.__BCAPI.supportedObj.platform.deviceInfo.isSupported;
			},
			
			/**
			 * Get device information
			 * 
		     * @param {function} cbSuccess - Function to be called upon success
		     * 		signature: function cbSuccess(deviceInfoObj)
		     * 					deviceInfoObj - contains:
		     * 							platform: device's operating system name (e.g. Android)
		     * 							version: operating system version (e.g. 2.1 in case of Android).
		     * 							uuid: device's Universally Unique Identifier
		     * 							language: ISO 639-1 language code (e.g. 'en' for English).
		     *  
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
			 * 
			 * @author Yoel Gluschnaider
			 */
			getDeviceInfo : function (cbSuccess, cbFail) 
			{
				rpcSender.invoke('conduitApi.platform.deviceInfo.getDeviceInfo', cbSuccess, cbFail);
			}
		},
	
		/**
		 * Geo Location API.
		 */
		geo: 
		{
			/**
			 * Check if the Geo Location API is supported.
			 * @return {boolean} true if it is supported and false otherwise.
			 */
			isSupported: function () 
			{
				return conduitApi.__BCAPI.supportedObj.platform.geo.isSupported;
			},
			
			/**
		     * Asynchronously aquires the current position.
			 *
		     * @param cbSuccess - Function to be called upon success
		     * 					  Signature: cbSuccess(position).
		     * 					  Param position contains:
		     * 						coords: A set of geographic coordinates
		     * 							latitude: Latitude in decimal degrees. (Number)
		     * 							longitude: Longitude in decimal degrees. (Number)
		     * 							altitude: Height of the position in meters above the ellipsoid. (Number)
		     * 							accuracy: Accuracy level of the latitude and longitude coordinates in meters. (Number)
		     * 							altitudeAccuracy: Accuracy level of the altitude coordinate in meters. (Number)
		     * 							heading: Direction of travel, specified in degrees counting clockwise relative to the true north. (Number)
		     * 							speed: Current ground speed of the device, specified in meters per second. (Number)
		     * 						timestamp: Creation timestamp for coords in milliseconds. (DOMTimeStamp)
		     * 
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail(error)
			 * 								Error: The error returned by the device - 
			 * 									code: One of the predefined error codes listed below:
			 * 										Error.PERMISSION_DENIED
			 * 										Error.POSITION_UNAVAILABLE
			 * 										Error.TIMEOUT
			 * 									message: Error message describing the details of the error encountered.
		     * @param options - Optional parameters to customize the retrieval of the geolocation.
		     * 					{ maximumAge: 3000, timeout: 5000, enableHighAccuracy: true };
		     *                  The supported options are:
		     *                      frequency: How often to retrieve the position in milliseconds. This option is not part of the W3C spec and will be removed in the future. maximumAge should be used instead. (Number) (Default: 10000)
		     *                      enableHighAccuracy: Provides a hint that the application would like to receive the best possible results. (Boolean)
		     *                      timeout: The maximum length of time (msec) that is allowed to pass from the call to geolocation.getCurrentPosition or geolocation.watchPosition until the corresponding geolocationSuccess callback is invoked. (Number)
		     *                      maximumAge: Accept a cached position whose age is no greater than the specified time in milliseconds. (Number)
		     *                      
		     *                  Android Quirks
		     *                  The Android 2.x simulators will not return a geolocation result unless the enableHighAccuracy option is set to true.
		     *                  { enableHighAccuracy: true }
			 * 
			 * @author Yoel Gluschnaider
		     */
	    	getLocation: function(cbSuccess, cbFail, options) 
			{
				rpcSender.invoke('conduitApi.platform.geo.getLocation', cbSuccess, cbFail, options);
			},
			
			/**
			 * Watches for changes to the device's current position.
		     * @param {function} cbSuccess - Function to be called upon success
		     * 		signature: function cbSuccess(geoHandler, position)
		     * 					geoHandle - to be used in stopGeoLocation.
		     * 					position - contains:
		     * 							coords: A set of geographic coordinates:
		     * 								latitude: Latitude in decimal degrees. (Number)
		     * 								longitude: Longitude in decimal degrees. (Number)
		     * 								altitude: Height of the position in meters above the ellipsoid. (Number)
		     * 								accuracy: Accuracy level of the latitude and longitude coordinates in meters. (Number)
		     *								altitudeAccuracy: Accuracy level of the altitude coordinate in meters. (Number)
		     * 								heading: Direction of travel, specified in degrees counting clockwise relative to the true north. (Number)
		     * 								speed: Current ground speed of the device, specified in meters per second. (Number)
		     * 								
		     * 							timestamp: Creation timestamp for coords in milliseconds. (DOMTimeStamp)  
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail(error)
			 * 								Error: The error returned by the device - 
			 * 									code: One of the predefined error codes listed below:
			 * 										Error.PERMISSION_DENIED
			 * 										Error.POSITION_UNAVAILABLE
			 * 										Error.TIMEOUT
			 * 									message: Error message describing the details of the error encountered.
			 * 
		     * @param options - Optional parameters to customize the retrieval of the geolocation.
		     * 					{ maximumAge: 3000, timeout: 5000, enableHighAccuracy: true };
		     *                  The supported options are:
		     *                      frequency: How often to retrieve the position in milliseconds. This option is not part of the W3C spec and will be removed in the future. maximumAge should be used instead. (Number) (Default: 10000)
		     *                      enableHighAccuracy: Provides a hint that the application would like to receive the best possible results. (Boolean)
		     *                      timeout: The maximum length of time (msec) that is allowed to pass from the call to geolocation.getCurrentPosition or geolocation.watchPosition until the corresponding geolocationSuccess callback is invoked. (Number)
		     *                      maximumAge: Accept a cached position whose age is no greater than the specified time in milliseconds. (Number)
		     *                      
		     *                  Android Quirks
		     *                  The Android 2.x simulators will not return a geolocation result unless the enableHighAccuracy option is set to true.
		     *                  { enableHighAccuracy: true }
			 */
			startGeoLocation : function (cbSuccess, cbFail, options) 
			{
				rpcSender.invoke('conduitApi.platform.geo.startGeoLocation', cbSuccess, cbFail, options);
			},
	
			/**
		     * @param {function} cbSuccess - Function to be called upon success
		     * 		signature: function cbSuccess()
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
			 * @param {Object} geoHandle - retrieved from the startGeoLocation function
			 */
			stopGeoLocation : function (cbSuccess, cbFail, geoHandle) 
			{
				rpcSender.invoke('conduitApi.platform.geo.stopGeoLocation', cbSuccess, cbFail, geoHandle);
			}
		},
	
		/**
		 * Network API.
		 */
		network: 
		{
			/**
			 * Check if the Network API is supported.
			 * @return {boolean} true if it is supported and false otherwise.
			 */
			isSupported: function () 
			{
				return conduitApi.__BCAPI.supportedObj.platform.network.isSupported;
			},
			
			/**
		     * Send http request
		     * @param cbSuccess - Function to be called upon success
		     * 			Signature: cbSuccess(data, requestMetaData)
		     * 					data - The data returned from the server, formatted according to the 'dataType' parameter.
		     * 					requestMetaData - same as the requestMetaData parameter.
			 * @param {function} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail(requestMetaData)
		     * @param sMethod - Method of request: GET, PUT. default is GET
		     * @param sUrl - Request URL
		     * @param sParams - Request body (for PUT)
		     * @param bAsync - Make asynchronous request
		     * @param sDataType - Type of response data. default is JSON
		     * @param sUser - User name for authenticated request
		     * @param sPassword - Password for authenticated request
		     * @param requestMetaData - Data to be passed back to callbacks
		     *
			 * @author Yoel Gluschnaider
		     */
			crossDomainHttpRequest: function (cbSuccess, cbFail, sMethod, sUrl, sParams, bAsync, sDataType, sUser, sPassword, requestMetaData) 
			{
				rpcSender.invoke('conduitApi.platform.network.crossDomainHttpRequest', cbSuccess, cbFail, sMethod, sUrl, sParams, bAsync, sDataType, sUser, sPassword, requestMetaData);
			}
		},
	
		/**
		 * Screen Orientation API.
		 */
		orientation: 
		{
			/**
			 * Check if the Screen Orientation API is supported.
			 * @return {boolean} true if it is supported and false otherwise.
			 */
			isSupported: function () 
			{
				return conduitApi.__BCAPI.supportedObj.platform.orientation.isSupported;
			},
			
			/**
		     * Get the current orientation
		     * @param cbSuccess - Function to be called upon success
		     * 					  Signature: cbSuccess(curOrientation).
		     * 					  Param position contains:
		     * 						curOrientation: signed integer of device orientation 
		     * 										degree from base can be: 0, 90, 180, -90
		     * 
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
			 *
			 * @author Yoel Gluschnaider
		     */ 
		    
			getOrientation: function (cbSuccess, cbFail) 
			{
				rpcSender.invoke('conduitApi.platform.orientation.getOrientation', cbSuccess, cbFail);
			},
			
			/**
		     * Lock device on the orientation
		 	 *
		     * @param cbSuccess - Function to be called upon success
		     * 					  Signature: function cbSuccess();
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
			 * @param degree - signed integer of device orientation degree from base
			 *				  can be: 0, 90, 180, -90
			 *
			 * @author Yoel Gluschnaider
		     */ 
			lockOrientation: function (cbSuccess, cbFail, degree) 
			{
				rpcSender.invoke('conduitApi.platform.orientation.lockOrientation', cbSuccess, cbFail, degree);
			},
			
			/**
		     * Unlock device on the orientation
		     * @param cbSuccess - Function to be called upon success
		     * 					  Signature: function cbSuccess();
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
		 	 *
			 * @author Yoel Gluschnaider
			 */ 
			unlockOrientation: function (cbSuccess, cbFail) 
			{
				rpcSender.invoke('conduitApi.platform.orientation.unlockOrientation', cbSuccess, cbFail);
			},
	
			/**
		     * Register event handler for orientation changed
			 *
		     * @param cbSuccess - Orientation change event handler
		     * 							Signature: cbOrientationChanged(degrees)
		     * 								degrees: 0, 90, 180 or -90.
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
			 *
			 * @author Yoel Gluschnaider
		     */
		    startOrientation: function (cbSuccess, cbFail) 
			{
				rpcSender.invoke('conduitApi.platform.orientation.startOrientation', cbSuccess, cbFail);
			},
			
			/**
		     * Unregister orientation changed event
			 * 
		     * @param cbSuccess - Function to be called upon success
		     * 					  Signature: function cbSuccess();
			 * @param {Object} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
			 */
			stopOrientation: function (cbSuccess, cbFail) 
			{
				rpcSender.invoke('conduitApi.platform.orientation.stopOrientation', cbSuccess, cbFail);
			}
		},
	
		/**
		 * Local Storage API.
		 */
		storage: 
		{
			/**
			 * Check if the accelerometer API is supported.
			 * @return {boolean} true if it is supported and false otherwise.
			 */
			isSupported: function () 
			{
				return conduitApi.__BCAPI.supportedObj.platform.storage.isSupported;
			},
	
		    /**
		     * Save a value for a key under an identifier
		     * @param {Function} cbSuccess - Function to be called upon success
		     * 					  		Signature: function cbSuccess();
			 * @param {Function} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
		     * @param key - Key name
		     * @param value - Value for the key
		     * 
		     * @return true if succeeded and false otherwise.
			 * @author Yoel Gluschnaider
		     */
			setItem: function (cbSuccess, cbFail, key, value) 
			{
				rpcSender.invoke('conduitApi.platform.storage.setItem', cbSuccess, cbFail, key, value);
			},
			
		    /**
		     * Get value for a key under an identifier
		     * 
		     * @param key - Key name
		     * @param {Function} cbSuccess - Function to be called upon success
		     * 					  		Signature: function cbSuccess(value);
			 * @param {Function} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
		     * @return {object} the value or null if none exists.
		     *
			 * @author Yoel Gluschnaider
		     */
			getItem: function (cbSuccess, cbFail, key)
			{
				rpcSender.invoke('conduitApi.platform.storage.getItem', cbSuccess, cbFail, key);
			},
			
		    /**
		     * Remove a key under an identifier
		     * @param {Function} cbSuccess - Function to be called upon success
		     * 					  		Signature: function cbSuccess();
			 * @param {Function} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
		     * @param key - Key name
		     * @return true if succeeded and false otherwise.
		     *
			 * @author Yoel Gluschnaider
		     */
			removeItem: function (cbSuccess, cbFail, key) 
			{
				rpcSender.invoke('conduitApi.platform.storage.removeItem', cbSuccess, cbFail, key);
			},
			
		    /**
		     * Clear the whole storage for an identifier
		     * @param {Function} cbSuccess - Function to be called upon success
		     * 					  		Signature: function cbSuccess();
			 * @param {Function} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
		     * @return true if succeeded and false otherwise.
		     *
			 * @author Yoel Gluschnaider
		     */
			clearStorage: function (cbSuccess, cbFail) 
			{
				rpcSender.invoke('conduitApi.platform.storage.clearStorage', cbSuccess, cbFail);
			}
		},
	
		/**
		 * Vibration API.
		 */
		vibrate: 
		{
			/**
			 * Check if the Vibration API is supported.
			 * @return {boolean} true if it is supported and false otherwise.
			 */
			isSupported: function () 
			{
				return conduitApi.__BCAPI.supportedObj.platform.vibrate.isSupported;
			},
	
		    /**
		     * Vibrate device for a specific time
			 *
		     * @param {Function} cbSuccess - Function to be called upon success
		     * 					  		Signature: function cbSuccess();
			 * @param {Function} cbFail - callback function in case the API callback failed.
			 * 							Signature: function cbFail()
			 * @param nTime - Time in miliseconds for the vibration
			 *					nTime range from 0 to 10000. default is 10
		     *
			 * @author Yoel Gluschnaider
		     */
			vibrateForTime: function (cbSuccess, cbFail, nTime) 
			{
				rpcSender.invoke('conduitApi.platform.vibrate.vibrateForTime', cbSuccess, cbFail, nTime);
			}
	
		}
		
	},
	
	analytics:
	{
		/**
		 * Check if the analytics API is supported.
		 * 
		 * @return {boolean} true if it is supported and false otherwise.
		 */
		isSupported: function () 
		{
			return conduitApi.__BCAPI.supportedObj.analytics.isSupported;
		},
		
		/**
		 * 
	     * Add event to analytics stack
		 *
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
	     * @param type - Type of events from usageEventTypeEnum
	     * @param meta - The event's meta data
	     * 
		 * @author Moshe Darzi
		 */
		addEvent: function (cbSuccess, cbFail, type, meta) 
		{
			rpcSender.invoke('conduitApi.analytics.addEvent', cbSuccess, cbFail, type, meta);
		}
	},
	
	log: 
	{
		/**
		 * Check if the log API is supported.
		 * 
		 * @return {boolean} true if it is supported and false otherwise.
		 */
		isSupported: function () 
		{
			return conduitApi.__BCAPI.supportedObj.log.isSupported;
		},
		
		/**
		 * Write a log message to the native logger.
		 * Verbosity - Info (if supported).
		 * 
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @param {Object} message the log message.
		 */
		info: function (cbSuccess, cbFail, message) 
		{
			rpcSender.invoke('conduitApi.log.info', cbSuccess, cbFail, message);
		},
		
		/**
		 * Write a log message to the native logger.
		 * Verbosity - warning (if supported).
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @param {Object} message the log message.
		 */
		warning: function (cbSuccess, cbFail, message) 
		{
			rpcSender.invoke('conduitApi.log.warning', cbSuccess, cbFail, message);
		},
		
		/**
		 * Write a log message to the native logger.
		 * Verbosity - error (if supported).
	     * @param cbSuccess - Function to be called upon success
	     * 					  Signature: function cbSuccess();
		 * @param cbFail - callback function in case the API callback failed.
		 *					  Signature: function cbFail()
		 * @param {Object} message the log message.
		 */
		error: function (cbSuccess, cbFail, message) 
		{
			rpcSender.invoke('conduitApi.log.error', cbSuccess, cbFail, message);
		}
	}
};


conduitApi.__BCAPI.app = (function () 
{
	var me = {};
	
	/**
	 * Check if the app API is supported.
	 * 
	 * @return {boolean} true if it is supported and false otherwise.
	 */
	me.isSupported = function () 
	{
		return conduitApi.__BCAPI.supportedObj.app.isSupported;
	};
	
	/**
	 * Hide info bar.
	 * 
	 * @param cbSuccess - Function to be called upon success
	 * 					  Signature: function cbSuccess();
	 * @param cbFail - callback function in case the API callback failed.
	 *					  Signature: function cbFail()
	 * @return {boolean} true if it is succeeded and false otherwise.
	 * 
	 * @author Moshe Darzi
	 */
	me.hideInfoBar = function (cbSuccess, cbFail) 
	{
		rpcSender.invoke('conduitApi.app.hideInfoBar', cbSuccess, cbFail);
	};
	
	/**
	 * Show info bar.
	 * 
	 * @param cbSuccess - Function to be called upon success
	 * 					  Signature: function cbSuccess();
	 * @param cbFail - callback function in case the API callback failed.
	 *					  Signature: function cbFail()
	 * @return {boolean} true if it is succeeded and false otherwise.
	 * 
	 * @author Moshe Darzi
	 */
	me.showInfoBar = function (cbSuccess, cbFail) 
	{
		rpcSender.invoke('conduitApi.app.showInfoBar', cbSuccess, cbFail);
	};
	
	/**
	 * Toggle visiblity of the info bar.
	 * 
	 * @param cbSuccess - Function to be called upon success
	 * 					  Signature: function cbSuccess();
	 * @param cbFail - callback function in case the API callback failed.
	 *					  Signature: function cbFail()
	 * @return {boolean} true if it is succeeded and false otherwise.
	 * 
	 * @author Moshe Darzi
	 */
	me.toggleInfoBar = function (cbSuccess, cbFail) 
	{
		rpcSender.invoke('conduitApi.app.toggleInfoBar', cbSuccess, cbFail);
	};
	
	/**
	 * Return the visiblity of the info bar.
	 * 
	 * @param cbSuccess - Function to be called upon success
	 * 					  Signature: function cbSuccess(visible);
	 * @param cbFail - callback function in case the API callback failed.
	 *					  Signature: function cbFail()
	 * @return true if visible, false otherwise
	 * 
	 * @author Moshe Darzi
	 */
	me.isInfoBarVisible = function (cbSuccess, cbFail)
	{
		rpcSender.invoke('conduitApi.app.isInfoBarVisible', cbSuccess, cbFail);
	};
	
	/**
	 * Hide navigation bar.
	 * 
	 * @param cbSuccess - Function to be called upon success
	 * 					  Signature: function cbSuccess();
	 * @param cbFail - callback function in case the API callback failed.
	 *					  Signature: function cbFail()
	 * @return {boolean} true if it is succeeded and false otherwise.
	 * 
	 * @author Moshe Darzi
	 */
	me.hideNavigationBar = function (cbSuccess, cbFail) 
	{
		rpcSender.invoke('conduitApi.app.hideNavigationBar', cbSuccess, cbFail);
	};
	
	/**
	 * Show navigation bar.
	 * 
	 * @param cbSuccess - Function to be called upon success
	 * 					  Signature: function cbSuccess();
	 * @param cbFail - callback function in case the API callback failed.
	 *					  Signature: function cbFail()
	 * @return {boolean} true if it is succeeded and false otherwise.
	 * 
	 * @author Moshe Darzi
	 */
	me.showNavigationBar = function (cbSuccess, cbFail) 
	{
		rpcSender.invoke('conduitApi.app.showNavigationBar', cbSuccess, cbFail);
	};
	
	/**
	 * Toggle visiblity of the navigation bar.
	 * 
	 * @param cbSuccess - Function to be called upon success
	 * 					  Signature: function cbSuccess();
	 * @param cbFail - callback function in case the API callback failed.
	 *					  Signature: function cbFail()
	 * @return {boolean} true if it is succeeded and false otherwise.
	 * 
	 * @author Moshe Darzi
	 */
	me.toggleNavigationBar = function (cbSuccess, cbFail) 
	{
		rpcSender.invoke('conduitApi.app.toggleNavigationBar', cbSuccess, cbFail);
	};
	
	/**
	 * Return the visiblity of the navigation bar.
	 * 
	 * @param cbSuccess - Function to be called upon success
	 * 					  Signature: function cbSuccess(visable);
	 * @param cbFail - callback function in case the API callback failed.
	 *					  Signature: function cbFail()
	 * @return true if visible, false otherwise
	 * 
	 * @author Moshe Darzi
	 */
	me.isNavigationBarVisible = function (cbSuccess, cbFail)
	{
		rpcSender.invoke('conduitApi.app.isNavigationBarVisible', cbSuccess, cbFail);
	};
	
	// the buttons call backs map.
	var buttonsCbsMap = {};
	// the handle of the btnClick event.
	var btnClickEventHandle;
	
	var buttonsCbsMapCount = 0;
	
	/**
	 * Add button to the info bar.
	 * 
	 * @param cbSuccess - Function to be called upon success
	 * 					  Signature: function cbSuccess(btnHandle);
	 * 						btnHandle - the button's handle for later removal.
	 * @param cbFail - callback function in case the API callback failed.
	 *					  Signature: function cbFail()
	 * @param {Object} cbButton - the callback of the button.
	 * @param {Object} buttonImage - the image of the button.
	 * @param {bool} addToRight - true indicates the button will be added to the right side,
	 * 							false will add to the left side
	 * 
	 * @return {Object} the button's handle for later removal or null if the addition failed.
	 * 
	 * @author Moshe Darzi
	 */
	me.addInfoBarButton = function (cbSuccess, cbFail, cbButton, buttonImage, addToRight) 
	{
		/**
		 * Inner event handler to send the click event to the relevant button.
		 * @param {Object} eventName - in this case 'btnClick'.
		 * @param {Object} data - the data of the click event.
		 */
		function btnClickEventHandler(eventName, data) 
		{
			// sanity check
			if (eventName !== 'btnClick') 
			{
				return;
			}
			buttonsCbsMap[''+data.btnId]();	
		}
		
		/**
		 * Callback when a button was added. saves it in the buttons map.
		 * @param {Object} buttonHandle - the button's handle.
		 */
		function innerCbSucces(buttonHandle) 
		{
			buttonsCbsMap[''+buttonHandle] = cbButton;
			cbSuccess(buttonHandle);
		}
		
		// if first button, register to the event.
		if (!btnClickEventHandle) 
		{
			btnClickEventHandle = eventsHandler.registerOnEvent('btnClick', btnClickEventHandler);
		}
		rpcSender.invoke('conduitApi.app.addInfoBarButton', innerCbSucces, cbFail, buttonImage, addToRight);
	};
	
	/**
	 * Remove button from the info bar.
	 * 
	 * @param cbSuccess - Function to be called upon success
	 * 					  Signature: function cbSuccess();
	 * @param cbFail - callback function in case the API callback failed.
	 *					  Signature: function cbFail()
	 * @param {Object} buttonHandle - the handle of the button.
	 * 
	 * @return {boolean} true if it is succeeded and false otherwise.
	 * 
	 * @author Moshe Darzi
	 */
	me.removeInfoBarButton = function (cbSuccess, cbFail, buttonHandle) 
	{
		delete buttonsCbsMap[''+buttonHandle];
		rpcSender.invoke('conduitApi.app.removeInfoBarButton', cbSuccess, cbFail, buttonHandle);
	};
	
	/**
	 * Remove all buttons from the info bar.
	 * 
	 * @param cbSuccess - Function to be called upon success
	 * 					  Signature: function cbSuccess();
	 * @param cbFail - callback function in case the API callback failed.
	 *					  Signature: function cbFail()
	 * @return {boolean} true if it is succeeded and false otherwise.
	 * 
	 * @author Moshe Darzi
	 */
	me.removeAllInfoBarButtons = function (cbSuccess, cbFail) 
	{
		rpcSender.invoke('conduitApi.app.removeAllInfoBarButtons', cbSuccess, cbFail);
	};
	
	/**
	 * 
	 * @param cbSuccess - Function to be called upon success
	 * 					  Signature: function cbSuccess();
	 * @param cbFail - callback function in case the API callback failed.
	 *					  Signature: function cbFail()
	 * @param {Object} args - contains the url and the title.
	 * 							{
	 * 								url: 'http://www.conduit.com',
	 * 								title: 'Conduit'
	 * 							}
	 * 
	 * @author Moshe Darzi
	 */
	me.openLink = function (cbSuccess, cbFail, args) 
	{
		rpcSender.invoke('conduitApi.app.openLink', cbSuccess, cbFail, args);
	};
	
	/**
	 * Closes the application.
	 * 
	 * @param cbSuccess - Function to be called upon success
	 * 					  Signature: function cbSuccess();
	 * @param cbFail - callback function in case the API callback failed.
	 *					  Signature: function cbFail()
	 * @author Moshe Darzi
	 */
	me.closeApp = function (cbSuccess, cbFail) 
	{
		rpcSender.invoke('conduitApi.app.closeApp', cbSuccess, cbFail);
	};
	
	return me;
	
})();

conduitApi.__BCAPI.UI = (function () 
{
	var me = {};

	/**
	 * Check if the UI API is supported.
	 * 
	 * @return {boolean} true if it is supported and false otherwise.
	 */
	me.isSupported = function () 
	{
		return conduitApi.__BCAPI.supportedObj.UI.isSupported;
	};
	
	/**
	 * Show an activity indicator.
	 * 
     * @param cbSuccess - Function to be called upon success
     * 					  Signature: function cbSuccess();
	 * @param cbFail - callback function in case the API callback failed.
	 *					  Signature: function cbFail()
	 * @param {Object} message - the message to be shown in the Activity Indicator.
	 */
	me.showActivityIndicator = function (cbSuccess, cbFail, message) 
	{
		rpcSender.invoke('conduitApi.UI.showActivityIndicator', cbSuccess, cbFail, message);
	};
	
	/**
	 * Dismiss the activity indicator.
	 * 
     * @param cbSuccess - Function to be called upon success
     * 					  Signature: function cbSuccess();
	 * @param cbFail - callback function in case the API callback failed.
	 *					  Signature: function cbFail()
	 */
	me.dismissActivityIndicator = function (cbSuccess, cbFail) 
	{
		rpcSender.invoke('conduitApi.UI.dismissActivityIndicator', cbSuccess, cbFail);
	};
	
	/**
	 * Show a dialog box.
	 * 
     * @param cbSuccess - Function to be called upon success
     * 					  Signature: function cbSuccess();
	 * @param cbFail - callback function in case the API callback failed.
	 *					  Signature: function cbFail()
	 * @param {Object} The dialog options contains:
	 * 						title: the title of the dialog.
	 * 						message: the message of the dialog.
	 * 						buttons: an array containing Button objects that consist of:
	 * 								text: the text of the button.
	 * 								cb: the callback to run when the button is pressed.
	 */
	me.showDialogBox = function (cbSuccess, cbFail, dialogOptions) 
	{
		// register on the event.
		var eventHandle = eventsHandler.registerOnEvent('dlgBtnClick', dlgBtnClickEventHandler);
		
		// handler of the dialog button click.
		function dlgBtnClickEventHandler(eventName, data) 
		{
			// call the relevant call back.
			dialogOptions.buttons[data.btnIndex].cb();
			// unregister from the event.
			eventsHandler.unregisterFromEvent('dlgBtnClick', eventHandle);
		}
		rpcSender.invoke('conduitApi.UI.showDialogBox', cbSuccess, cbFail, dialogOptions);
	};
	
	return me;
})();

conduitApi.__BCAPI.platform.pushNotification = (function () 
{
	var me = {};
	
	var _pushNotifEventHandle;
	
	/**
	 * Check if the Push Notification API is supported.
	 * @return {boolean} true if it is supported and false otherwise.
	 */
	me.isSupported = function () 
	{
		return conduitApi.__BCAPI.supportedObj.platform.pushNotification.isSupported;
	};

	/**
     * Register component for push notification
     * @param cbSuccess - Function to be called when the user chose to view the notification.
	 * @param {Object} cbFail - callback function in case the API callback failed.
	 * 							Signature: function cbFail()
	 *
	 * @author Yoel Gluschnaider
     */
    me.registerPushNotification = function (cbSuccess, cbFail, cbEventHandler)
	{
		function pushNotificationHandler(eventaName, data) 
		{
			cbEventHandler(data.message);
		}
		_pushNotifEventHandle = eventsHandler.registerOnEvent('pushNotif', pushNotificationHandler);
		rpcSender.invoke('conduitApi.platform.pushNotification.registerPushNotification', cbSuccess, cbFail);
	};
	
	/**
     * Remove registeration for a component
     * @param {Function} cbSuccess - Function to be called upon success
     * 					  		Signature: function cbSuccess();
	 * @param {Function} cbFail - callback function in case the API callback failed.
	 * 							Signature: function cbFail()
	 *
	 * @author Yoel Gluschnaider
     */
    me.unregisterPushNotification = function (cbSuccess, cbFail) 
	{
		eventsHandler.unregisterFromEvent('pushNotif', _pushNotifEventHandle);
		rpcSender.invoke('conduitApi.platform.pushNotification.unregisterPushNotification', cbSuccess, cbFail);
	};
	
	return me;
})();

conduitApi.__BCAPI.platform.gestures = (function () 
{
	var me = {};
	
	var shageEventHandle;
	
	/**
	 * Check if the Gestures API is supported.
	 * @return {boolean} true if it is supported and false otherwise.
	 */
	me.isSupported = function () 
	{
		return conduitApi.__BCAPI.supportedObj.platform.gestures.isSupported;
	};
	
	/**
     * Register component for shake gesture
	 *
     * @param cbSuccess - Function to be called upon success
     * 					  Signature: function cbSuccess();
	 * @param cbFail - callback function in case the API callback failed.
	 *					  Signature: function cbFail()
     * @param cbEventHandler - Function to be called when the user shake the device
	 *
	 * @author Moshe Darzi
     */
	me.registerShake = function (cbSuccess, cbFail, cbEventHandler) 
	{
		function shakeEventHandler(eventName, data) 
		{
			cbEventHandler();
		}
		shageEventHandle = eventsHandler.registerOnEvent('shakeEvent', shakeEventHandler);
		
		rpcSender.invoke('conduitApi.platform.gestures.registerShake', cbSuccess, cbFail);
	};
	
	/**
     * Remove registeration for a component
	 *
     * @param cbSuccess - Function to be called upon success
     * 					  Signature: function cbSuccess();
	 * @param cbFail - callback function in case the API callback failed.
	 *					  Signature: function cbFail()
	 * @author Moshe Darzi
     */
	me.unregisterShake = function (cbSuccess, cbFail) 
	{
		eventsHandler.unregisterFromEvent('shakeEvent', shageEventHandle);
		rpcSender.invoke('conduitApi.platform.gestures.unregisterShake', cbSuccess, cbFail);
	};
	
	return me;
})();
